# encoding: UTF-8
=begin
Created on 2017-03-17

@author: Nikolay Moskvin <moskvin@sibext.com>

=end

module Resper
  
  VERSION = '0.2.3'
  
  require 'json'
  require 'string'

  class Parser

    @@ANDROID_SCALES = [
      {s: 1.0, d: 160, o: 'mdpi'},
      {s: 1.5, d: 240, o: 'hdpi'},
      {s: 2.0, d: 320, o: 'xhdpi'},
      {s: 3.0, d: 480, o: 'xxhdpi'},
      {s: 4.0, d: 640, o: 'xxxhdpi'},
    ]

    @@IOS_UNIVERSAL_SCALES = [
      {s: 1.0, d: 160, o: ''},
      {s: 2.0, d: 320, o: '@2x'},
      {s: 3.0, d: 480, o: '@3x'},
    ]

    def initialize type, input, width, height, output, format
      @type = type
      @input = input
      @width = width
      @height = height
      @output = output
      @format = format
    end

    def parse_resources!
      if @type == :all || @type == :android
	      @@ANDROID_SCALES.each do |scale|
		file = for_android w: scale[:s]*@width, h: scale[:s]*@height,
				   d: scale[:d], o: "drawable-#{scale[:o]}"
		reduce_size file
	      end
      end

      if @type == :all || @type == :ios
	      @@IOS_UNIVERSAL_SCALES.each do |scale|
		file = for_ios scale[:s], scale[:o], scale[:d]
		reduce_size file
	      end
	      generate_universal_contents 
      end
      if @type == :all || @type == :react
	      @@IOS_UNIVERSAL_SCALES.each do |scale|
		file = for_react scale[:s], scale[:o], scale[:d]
		reduce_size file
	      end
      end

    end
    
    private

    def for_android w:, h:, d:, o:
      out = File.join(android_output_path, o)
      convert width: w, height: h, density: d, to: out, name: android_image_name
    end

    def for_ios factor, postfix, density
      convert width: @width*factor, height: @height*factor, 
							density: density, to: ios_output_path, name: ios_image_name(postfix)
    end

    def for_react factor, postfix, density
      path = @output[2]
      name = "#{react_image_name_without_ext}#{postfix}.png"
      name = "#{react_image_name_without_ext}.png" if factor == 1
       
      convert width: @width*factor, height: @height*factor, density: density, to: path, name: name
    end
  
    def generate_universal_contents
      images = @@IOS_UNIVERSAL_SCALES.map do |scale|
        {
          idiom: 'universal',
					filename: ios_image_name(scale[:o]),
          scale: "#{scale[:s].to_i}x"
        }
      end
      meta = {
        images: images,
        info: {
          version: 1,
					author: 'xcode'
				}
      }
      File.open(File.join(ios_output_path, 'Contents.json'),'w') do |f|
        f.write(JSON.pretty_generate(meta))
      end
    end
    
    def convert width:, height:, density:, to:, name:
      file_output = File.join(to, "#{name}")
      FileUtils.mkdir_p to
      `convert -size '#{width}x#{height}' -verbose -background none #{@format}:#{@input} #{file_output}`
      file_output
    end

    def reduce_size png_path
      system('pngquant', "#{png_path}", '--ext=.png', '--force')
    end
    
    def ios_output_path
      File.join(@output[1], "#{ios_image_name_without_ext}.imageset")
    end

    def ios_image_name postfix
      "#{ios_image_name_without_ext}#{postfix}.png"
    end

    def ios_image_name_without_ext
      name = File.basename(@input, ".*").underscore
      name.split('_').map{|e| e.capitalize}.join
    end

    def react_image_name_without_ext
      name = File.basename(@input, ".*").underscore
      name.gsub "_", "-"
    end

    def android_output_path
      @output[0]
    end

    def android_image_name
			name = File.basename(@input, ".*")
      "#{name.underscore}.png"
    end

  end

end
